﻿/*
One of those Weeks Episode 38: The Landing Bay
Copyright (c) 2008,2011 Blazing Games Inc.
Code written by Billy D. Spelchan

This program is free software; you can redistribute it and/or modify it under 
the terms of the GNU General Public License as published by the Free Software 
Foundation; either version 2 of the License, or (at your option) any later 
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program; if not, write to the Free Software Foundation, Inc.,
59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.ootw.ep38
{
	// Adobe Imports
	import flash.display.MovieClip;
	import flash.display.SimpleButton;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import flash.ui.Keyboard;
	import flash.utils.*;

	public class OotW38Manager
	extends MovieClip
	{
// ***** CONSTANTS ***********************************************************

		public static const GIVE_UP_EVENT:String = "PlayerGaveUp";
		public static const WIN_GAME_EVENT:String = "PlayerWonGame";

		protected static const ROTATION:Number = 200.0;
		protected static const ACCELERATION:Number = 50;
		protected static const MAX_SPEED:Number = 100;

		private static const WALLS_X = -95;
		private static const WALLS_Y = -220;
		
// ***** VARIABLES ***********************************************************

		protected var _xspeed:Number;
		protected var _yspeed:Number;

		protected var _lastRedraw:int;
		protected var _turningLeft:Boolean;
		protected var _turningRight:Boolean;
		protected var _accelerating:Boolean;
		protected var _landed:Boolean;
		protected var _collideCount:int;

		// note that all _*_movie, _*_btn and _*_txt variables are in movie

// ***** CONSTRUCTOR *********************************************************

		public function OotW38Manager()
		{
			_lastRedraw = getTimer();
			stage.addEventListener(KeyboardEvent.KEY_DOWN, handleKeyDown); 
			stage.addEventListener(KeyboardEvent.KEY_UP, handleKeyUp); 
			stage.addEventListener(Event.ENTER_FRAME, handleFrame);
			_yes_btn.addEventListener(flash.events.MouseEvent.CLICK, handleTryAgain);
			_no_btn.addEventListener(flash.events.MouseEvent.CLICK, handleGiveUp);
			restartGame();
		}

// ***** Functions - Event handlers ******************************************

		public function handleKeyDown(evt:KeyboardEvent)
		{
			var kc:uint = evt.keyCode;
			
			switch (kc) {
				case Keyboard.LEFT:
					_turningLeft = true;
					break;
				case Keyboard.RIGHT:
					_turningRight = true;
					break;
				case Keyboard.UP:
					_accelerating = true;
					break;
			}
		}

		public function handleKeyUp(evt:KeyboardEvent)
		{
			var kc:uint = evt.keyCode;
			
			switch (kc) {
				case Keyboard.LEFT:
					_turningLeft = false;
					break;
				case Keyboard.RIGHT:
					_turningRight = false;
					break;
				case Keyboard.UP:
					_accelerating = false;
					break;
			}
		}

		public function handleFrame(evt:Event)
		{
			var curtime:int = getTimer();
			var delta:Number = Math.min(1.0, 
					Number(curtime - _lastRedraw)/1000);
			_lastRedraw = curtime;
			if (_turningLeft)
				turnLeft(delta);
			if (_turningRight)
				turnRight(delta);
			if (_accelerating)
				accelerate(delta);
			updatePosition(delta);
		}

		public function handleTryAgain(evt:Event)
		{
			restartGame();
		}

		public function handleGiveUp(evt:Event)
		{
			var evnt:Event = new Event(GIVE_UP_EVENT);
			this.dispatchEvent(evnt);
		}

// ***** Functions - General control *****************************************

		public function restartGame()
		{
			_hitmap_movie.x = WALLS_X;
			_hitmap_movie.y = WALLS_Y;
			_landingbay_movie.x = WALLS_X;
			_landingbay_movie.y = WALLS_Y;
			_xspeed = 0;
			_yspeed = 0;
			_lose_txt.visible = false;
			_yes_btn.visible = false;
			_no_btn.visible = false;
			_landed = false;
			_spaceship_movie.restoreShip();
			_collideCount = 0;
		}

		public function turnLeft(delta:Number)
		{
			_spaceship_movie.rotation -= (delta * ROTATION);
		}

		public function turnRight(delta:Number)
		{
			_spaceship_movie.rotation += (delta * ROTATION);
		}

		public function accelerate(delta:Number)
		{
			var rad:Number = _spaceship_movie.rotation * Math.PI/180;

			_xspeed = Math.max(-MAX_SPEED, 
					Math.min(MAX_SPEED, 
					_xspeed + delta * ACCELERATION * Math.sin(rad)));
			_yspeed = Math.max(-MAX_SPEED,
					Math.min(MAX_SPEED, 
					_yspeed + delta * ACCELERATION * -Math.cos(rad)));
			// trace ("speed: " + _xspeed + "," + _yspeed);
		}

		public function updatePosition(delta)
		{
			_hitmap_movie.x -= _xspeed * delta;
			_hitmap_movie.y -= _yspeed * delta;
			_landingbay_movie.x = _hitmap_movie.x;
			_landingbay_movie.y = _hitmap_movie.y;
			if (_hitmap_movie.hitTestPoint(320,240,true))
				collision();
			if (_landingbay_movie._target_movie.hitTestPoint(320,240,true)) {
				// trace("Inside landing target");
				if ( (Math.abs(_xspeed) < 5) && (Math.abs(_yspeed) < 5) )
				{
					_xspeed = 0;
					_yspeed = 0;
					if (!_landed) {
						_landed = true;
						trace ("Win event being sent");
						var evnt:Event = new Event(WIN_GAME_EVENT);
						this.dispatchEvent(evnt);
					}
				}
			}
		}
		
		public function collision()
		{
			_xspeed = 0;
			_yspeed = 0;
			if (_collideCount > 2)
				return;

			// avoid automatic collision when restarting
			++_collideCount;
			if (_collideCount < 2) 
				return;
				
			trace("Collision detected");
			_spaceship_movie.explodeShip();
			_lose_txt.visible = true;
			_yes_btn.visible = true;
			_no_btn.visible = true;
		}
	} // end class
} // end package
